@@ -1,17 +1,24 @@
+# -*- coding: utf-8 -*-
 '''
 Support for Linux LVM2
 '''
 
+# Import python libs
+import os.path
+
 # Import salt libs
 import salt.utils
 
+# Define the module's virtual name
+__virtualname__ = 'lvm'
+
 
 def __virtual__():
     '''
     Only load the module if lvm is installed
     '''
     if salt.utils.which('lvm'):
-        return 'lvm'
+        return __virtualname__
     return False
 
 
@@ -19,7 +26,9 @@ def version():
     '''
     Return LVM version from lvm version
 
-    CLI Example::
+    CLI Example:
+
+    .. code-block:: bash
 
         salt '*' lvm.version
     '''
@@ -33,7 +42,9 @@ def fullversion():
     '''
     Return all version info from lvm version
 
-    CLI Example::
+    CLI Example:
+
+    .. code-block:: bash
 
         salt '*' lvm.fullversion
     '''
@@ -49,14 +60,19 @@ def fullversion():
 def pvdisplay(pvname=''):
     '''
     Return information about the physical volume(s)
-    CLI Examples::
+
+    CLI Examples:
+
+    .. code-block:: bash
 
         salt '*' lvm.pvdisplay
         salt '*' lvm.pvdisplay /dev/md0
     '''
     ret = {}
-    cmd = 'pvdisplay -c {0}'.format(pvname)
-    cmd_ret = __salt__['cmd.run_all'](cmd)
+    cmd = ['pvdisplay', '-c']
+    if pvname:
+        cmd.append(pvname)
+    cmd_ret = __salt__['cmd.run_all'](cmd, python_shell=False)
 
     if cmd_ret['retcode'] != 0:
         return {}
@@ -84,14 +100,19 @@ def pvdisplay(pvname=''):
 def vgdisplay(vgname=''):
     '''
     Return information about the volume group(s)
-    CLI Examples::
+
+    CLI Examples:
+
+    .. code-block:: bash
 
         salt '*' lvm.vgdisplay
         salt '*' lvm.vgdisplay nova-volumes
     '''
     ret = {}
-    cmd = 'vgdisplay -c {0}'.format(vgname)
-    cmd_ret = __salt__['cmd.run_all'](cmd)
+    cmd = ['vgdisplay', '-c']
+    if vgname:
+        cmd.append(vgname)
+    cmd_ret = __salt__['cmd.run_all'](cmd, python_shell=False)
 
     if cmd_ret['retcode'] != 0:
         return {}
@@ -124,14 +145,19 @@ def vgdisplay(vgname=''):
 def lvdisplay(lvname=''):
     '''
     Return information about the logical volume(s)
-    CLI Examples::
+
+    CLI Examples:
+
+    .. code-block:: bash
 
         salt '*' lvm.lvdisplay
         salt '*' lvm.lvdisplay /dev/vg_myserver/root
     '''
     ret = {}
-    cmd = 'lvdisplay -c {0}'.format(lvname)
-    cmd_ret = __salt__['cmd.run_all'](cmd)
+    cmd = ['lvdisplay', '-c']
+    if lvname:
+        cmd.append(lvname)
+    cmd_ret = __salt__['cmd.run_all'](cmd, python_shell=False)
 
     if cmd_ret['retcode'] != 0:
         return {}
@@ -161,7 +187,9 @@ def pvcreate(devices, **kwargs):
     '''
     Set a physical device to be used as an LVM physical volume
 
-    CLI Examples::
+    CLI Examples:
+
+    .. code-block:: bash
 
         salt mymachine lvm.pvcreate /dev/sdb1,/dev/sdb2
         salt mymachine lvm.pvcreate /dev/sdb1 dataalignmentoffset=7s
@@ -171,15 +199,36 @@ def pvcreate(devices, **kwargs):
 
     cmd = 'pvcreate'
     for device in devices.split(','):
+        if not os.path.exists(device):
+            return '{0} does not exist'.format(device)
         cmd += ' {0}'.format(device)
     valid = ('metadatasize', 'dataalignment', 'dataalignmentoffset',
              'pvmetadatacopies', 'metadatacopies', 'metadataignore',
              'restorefile', 'norestorefile', 'labelsector',
              'setphysicalvolumesize')
-    for var in kwargs.keys():
+    for var in kwargs:
         if kwargs[var] and var in valid:
             cmd += ' --{0} {1}'.format(var, kwargs[var])
-    out = __salt__['cmd.run'](cmd).splitlines()
+    out = __salt__['cmd.run'](cmd, python_shell=False).splitlines()
+    return out[0]
+
+
+def pvremove(devices):
+    '''
+    Remove a physical device being used as an LVM physical volume
+
+    CLI Examples:
+
+    .. code-block:: bash
+
+        salt mymachine lvm.pvremove /dev/sdb1,/dev/sdb2
+    '''
+    cmd = 'pvremove -y'
+    for device in devices.split(','):
+        if not __salt__['lvm.pvdisplay'](device):
+            return '{0} is not a physical volume'.format(device)
+        cmd += ' {0}'.format(device)
+    out = __salt__['cmd.run'](cmd, python_shell=False).splitlines()
     return out[0]
 
 
@@ -187,7 +236,9 @@ def vgcreate(vgname, devices, **kwargs):
     '''
     Create an LVM volume group
 
-    CLI Examples::
+    CLI Examples:
+
+    .. code-block:: bash
 
         salt mymachine lvm.vgcreate my_vg /dev/sdb1,/dev/sdb2
         salt mymachine lvm.vgcreate my_vg /dev/sdb1 clustered=y
@@ -200,34 +251,65 @@ def vgcreate(vgname, devices, **kwargs):
         cmd += ' {0}'.format(device)
     valid = ('clustered', 'maxlogicalvolumes', 'maxphysicalvolumes',
              'vgmetadatacopies', 'metadatacopies', 'physicalextentsize')
-    for var in kwargs.keys():
+    for var in kwargs:
         if kwargs[var] and var in valid:
             cmd += ' --{0} {1}'.format(var, kwargs[var])
-    out = __salt__['cmd.run'](cmd).splitlines()
+    out = __salt__['cmd.run'](cmd, python_shell=False).splitlines()
     vgdata = vgdisplay(vgname)
     vgdata['Output from vgcreate'] = out[0].strip()
     return vgdata
 
 
-def lvcreate(lvname, vgname, size=None, extents=None, pv=''):
+def lvcreate(lvname, vgname, size=None, extents=None, snapshot=None, pv=None, **kwargs):
     '''
     Create a new logical volume, with option for which physical volume to be used
 
-    CLI Examples::
+    CLI Examples:
+
+    .. code-block:: bash
 
         salt '*' lvm.lvcreate new_volume_name vg_name size=10G
-        salt '*' lvm.lvcreate new_volume_name vg_name extents=100 /dev/sdb
+        salt '*' lvm.lvcreate new_volume_name vg_name extents=100 pv=/dev/sdb
+        salt '*' lvm.lvcreate new_snapshot    vg_name snapshot=volume_name size=3G
     '''
     if size and extents:
-        return 'Error: Please specify only size or extents'
+        return 'Error: Please specify only one of size or extents'
+
+    valid = ('activate', 'chunksize', 'contiguous', 'discards', 'stripes',
+             'stripesize', 'minor', 'persistent', 'mirrors', 'noudevsync',
+             'monitor', 'ignoremonitoring', 'permission', 'poolmetadatasize',
+             'readahead', 'regionsize', 'thin', 'thinpool', 'type', 'virtualsize',
+             'zero',)
+    no_parameter = ('noudevsync', 'ignoremonitoring', )
+
+    extra_arguments = []
+    if kwargs:
+        for k, v in kwargs.iteritems():
+            if k in no_parameter:
+                extra_arguments.append('--{0}'.format(k))
+            elif k in valid:
+                extra_arguments.extend(['--{0}'.format(k), '{0}'.format(v)])
+
+    cmd = [salt.utils.which('lvcreate'), '-n', lvname]
+
+    if snapshot:
+        cmd.extend(['-s', vgname + '/' + snapshot])
+    else:
+        cmd.append(vgname)
 
     if size:
-        cmd = 'lvcreate -n {0} {1} -L {2} {3}'.format(lvname, vgname, size, pv)
+        cmd.extend(['-L', '{0}'.format(size)])
     elif extents:
-        cmd = 'lvcreate -n {0} {1} -l {2} {3}'.format(lvname, vgname, extents, pv)
+        cmd.extend(['-l', '{0}'.format(extents)])
     else:
         return 'Error: Either size or extents must be specified'
-    out = __salt__['cmd.run'](cmd).splitlines()
+
+    if pv:
+        cmd.append(pv)
+    if extra_arguments:
+        cmd.extend(extra_arguments)
+
+    out = __salt__['cmd.run'](cmd, python_shell=False).splitlines()
     lvdev = '/dev/{0}/{1}'.format(vgname, lvname)
     lvdata = lvdisplay(lvdev)
     lvdata['Output from lvcreate'] = out[0].strip()
@@ -238,13 +320,15 @@ def vgremove(vgname):
     '''
     Remove an LVM volume group
 
-    CLI Examples::
+    CLI Examples:
+
+    .. code-block:: bash
 
         salt mymachine lvm.vgremove vgname
         salt mymachine lvm.vgremove vgname force=True
     '''
     cmd = 'vgremove -f {0}'.format(vgname)
-    out = __salt__['cmd.run'](cmd)
+    out = __salt__['cmd.run'](cmd, python_shell=False)
     return out.strip()
 
 
@@ -252,10 +336,12 @@ def lvremove(lvname, vgname):
     '''
     Remove a given existing logical volume from a named existing volume group
 
-    CLI Example::
+    CLI Example:
+
+    .. code-block:: bash
 
         salt '*' lvm.lvremove lvname vgname force=True
     '''
     cmd = 'lvremove -f {0}/{1}'.format(vgname, lvname)
-    out = __salt__['cmd.run'](cmd)
+    out = __salt__['cmd.run'](cmd, python_shell=False)
     return out.strip()
