@@ -5,11 +5,21 @@ Windows Service module.
 
 # Import python libs
 import salt.utils
+import time
+import logging
+try:
+    from shlex import quote as _cmd_quote  # pylint: disable=E0611
+except ImportError:
+    from pipes import quote as _cmd_quote
+
+log = logging.getLogger(__name__)
 
 # Define the module's virtual name
 __virtualname__ = 'service'
 
 BUFFSIZE = 5000
+SERVICE_STOP_DELAY_SECONDS = 15
+SERVICE_STOP_POLL_MAX_ATTEMPTS = 5
 
 
 def __virtual__():
@@ -31,7 +41,9 @@ def has_powershell():
 
         salt '*' service.has_powershell
     '''
-    return 'powershell' in __salt__['cmd.run']('where powershell', output_loglevel='debug')
+    return 'powershell' in __salt__['cmd.run'](
+            ['where', 'powershell'], python_shell=False
+        )
 
 
 def get_enabled():
@@ -47,13 +59,13 @@ def get_enabled():
 
     if has_powershell():
         cmd = 'Get-WmiObject win32_service | where {$_.startmode -eq "Auto"} | select-object name'
-        lines = __salt__['cmd.run'](cmd, shell='POWERSHELL', output_loglevel='debug').splitlines()
+        lines = __salt__['cmd.run'](cmd, shell='POWERSHELL', python_shell=True).splitlines()
         return sorted([line.strip() for line in lines[3:]])
     else:
         ret = set()
         services = []
-        cmd = 'sc query type= service state= all bufsize= {0}'.format(BUFFSIZE)
-        lines = __salt__['cmd.run'](cmd, output_loglevel='debug').splitlines()
+        cmd = ['sc', 'query', 'type=', 'service', 'state=', 'all', 'bufsize=', str(BUFFSIZE)]
+        lines = __salt__['cmd.run'](cmd, python_shell=False).splitlines()
         for line in lines:
             if 'SERVICE_NAME:' in line:
                 comps = line.split(':', 1)
@@ -61,8 +73,8 @@ def get_enabled():
                     continue
                 services.append(comps[1].strip())
         for service in services:
-            cmd2 = 'sc qc "{0}" {1}'.format(service, BUFFSIZE)
-            lines = __salt__['cmd.run'](cmd2, output_loglevel='debug').splitlines()
+            cmd2 = ['sc', 'qc', service, str(BUFFSIZE)]
+            lines = __salt__['cmd.run'](cmd2, python_shell=False).splitlines()
             for line in lines:
                 if 'AUTO_START' in line:
                     ret.add(service)
@@ -81,13 +93,13 @@ def get_disabled():
     '''
     if has_powershell():
         cmd = 'Get-WmiObject win32_service | where {$_.startmode -ne "Auto"} | select-object name'
-        lines = __salt__['cmd.run'](cmd, shell='POWERSHELL', output_loglevel='debug').splitlines()
+        lines = __salt__['cmd.run'](cmd, shell='POWERSHELL', python_shell=True).splitlines()
         return sorted([line.strip() for line in lines[3:]])
     else:
         ret = set()
         services = []
-        cmd = 'sc query type= service state= all bufsize= {0}'.format(BUFFSIZE)
-        lines = __salt__['cmd.run'](cmd, output_loglevel='debug').splitlines()
+        cmd = ['sc', 'query', 'type=', 'service', 'state=', 'all', 'bufsize=', str(BUFFSIZE)]
+        lines = __salt__['cmd.run'](cmd, python_shell=False).splitlines()
         for line in lines:
             if 'SERVICE_NAME:' in line:
                 comps = line.split(':', 1)
@@ -95,8 +107,8 @@ def get_disabled():
                     continue
                 services.append(comps[1].strip())
         for service in services:
-            cmd2 = 'sc qc "{0}" {1}'.format(service, BUFFSIZE)
-            lines = __salt__['cmd.run'](cmd2, output_loglevel='debug').splitlines()
+            cmd2 = ['sc', 'qc', service, BUFFSIZE]
+            lines = __salt__['cmd.run'](cmd2, python_shell=False).splitlines()
             for line in lines:
                 if 'DEMAND_START' in line:
                     ret.add(service)
@@ -131,7 +143,7 @@ def missing(name):
 
         salt '*' service.missing <service name>
     '''
-    return not name in get_all()
+    return name not in get_all()
 
 
 def get_all():
@@ -170,8 +182,8 @@ def get_service_name(*args):
     ret = {}
     services = []
     display_names = []
-    cmd = 'sc query type= service state= all bufsize= {0}'.format(BUFFSIZE)
-    lines = __salt__['cmd.run'](cmd, output_loglevel='debug').splitlines()
+    cmd = ['sc', 'query', 'type=', 'service', 'state=', 'all', 'bufsize=', str(BUFFSIZE)]
+    lines = __salt__['cmd.run'](cmd, python_shell=False).splitlines()
     for line in lines:
         if 'SERVICE_NAME:' in line:
             comps = line.split(':', 1)
@@ -205,8 +217,8 @@ def start(name):
 
         salt '*' service.start <service name>
     '''
-    cmd = 'net start "{0}"'.format(name)
-    return not __salt__['cmd.retcode'](cmd)
+    cmd = ['net', 'start', name]
+    return not __salt__['cmd.retcode'](cmd, python_shell=False)
 
 
 def stop(name):
@@ -219,8 +231,25 @@ def stop(name):
 
         salt '*' service.stop <service name>
     '''
-    cmd = 'net stop "{0}"'.format(name)
-    return not __salt__['cmd.retcode'](cmd)
+    # net stop issues a stop command and waits briefly (~30s), but will give
+    # up if the service takes too long to stop with a misleading
+    # "service could not be stopped" message and RC 0.
+
+    cmd = ['net', 'stop', name]
+    res = __salt__['cmd.run'](cmd, python_shell=False)
+    if 'service was stopped' in res:
+        return True
+
+    # we requested a stop, but the service is still thinking about it.
+    # poll for the real status
+    for attempt in range(SERVICE_STOP_POLL_MAX_ATTEMPTS):
+        if not status(name):
+            return True
+        log.debug('Waiting for %s to stop', name)
+        time.sleep(SERVICE_STOP_DELAY_SECONDS)
+
+    log.warning('Giving up on waiting for service `%s` to stop', name)
+    return False
 
 
 def restart(name):
@@ -233,8 +262,10 @@ def restart(name):
 
         salt '*' service.restart <service name>
     '''
-    stop(name)
-    return start(name)
+    if has_powershell():
+        cmd = 'Restart-Service {0}'.format(_cmd_quote(name))
+        return not __salt__['cmd.retcode'](cmd, shell='powershell', python_shell=True)
+    return stop(name) and start(name)
 
 
 def status(name, sig=None):
@@ -249,8 +280,8 @@ def status(name, sig=None):
 
         salt '*' service.status <service name> [service signature]
     '''
-    cmd = 'sc query "{0}" {1}'.format(name, BUFFSIZE)
-    statuses = __salt__['cmd.run'](cmd, output_loglevel='debug').splitlines()
+    cmd = ['sc', 'query', name]
+    statuses = __salt__['cmd.run'](cmd, python_shell=False).splitlines()
     for line in statuses:
         if 'RUNNING' in line:
             return True
@@ -269,8 +300,8 @@ def getsid(name):
 
         salt '*' service.getsid <service name>
     '''
-    cmd = 'sc showsid "{0}"'.format(name)
-    lines = __salt__['cmd.run'](cmd, output_loglevel='debug').splitlines()
+    cmd = ['sc', 'showsid', name]
+    lines = __salt__['cmd.run'](cmd, python_shell=False).splitlines()
     for line in lines:
         if 'SERVICE SID:' in line:
             comps = line.split(':', 1)
@@ -290,8 +321,8 @@ def enable(name, **kwargs):
 
         salt '*' service.enable <service name>
     '''
-    cmd = 'sc config "{0}" start= auto'.format(name)
-    return not __salt__['cmd.retcode'](cmd)
+    cmd = ['sc', 'config', name, 'start=', 'auto']
+    return not __salt__['cmd.retcode'](cmd, python_shell=False)
 
 
 def disable(name, **kwargs):
@@ -304,8 +335,8 @@ def disable(name, **kwargs):
 
         salt '*' service.disable <service name>
     '''
-    cmd = 'sc config "{0}" start= demand'.format(name)
-    return not __salt__['cmd.retcode'](cmd)
+    cmd = ['sc', 'config', name, 'start=', 'disabled']
+    return not __salt__['cmd.retcode'](cmd, python_shell=False)
 
 
 def enabled(name):
