@@ -121,7 +121,9 @@ def decrypt_object(o, gpg):
             o[k] = decrypt_object(v, gpg)
         return o
     elif isinstance(o, list):
-        return [decrypt_object(e, gpg) for e in o]
+        for number, value in enumerate(o):
+            o[number] = decrypt_object(value, gpg)
+        return o
     else:
         return o
 
@@ -133,11 +135,10 @@ def render(data, saltenv='base', sls='', argline='', **kwargs):
     '''
     if not HAS_GPG:
         raise SaltRenderError('GPG unavailable')
-    if isinstance(__salt__, dict):
-        if 'config.get' in __salt__:
-            homedir = __salt__['config.get']('gpg_keydir', DEFAULT_GPG_KEYDIR)
-        else:
-            homedir = __opts__.get('gpg_keydir', DEFAULT_GPG_KEYDIR)
-        log.debug('Reading GPG keys from: {0}'.format(homedir))
+    if 'config.get' in __salt__:
+        homedir = __salt__['config.get']('gpg_keydir', DEFAULT_GPG_KEYDIR)
+    else:
+        homedir = __opts__.get('gpg_keydir', DEFAULT_GPG_KEYDIR)
+    log.debug('Reading GPG keys from: {0}'.format(homedir))
     gpg = gnupg.GPG(gnupghome=homedir)
     return decrypt_object(data, gpg)
