@@ -9,31 +9,34 @@ rc scripts, services can be defined as running or dead.
 .. code-block:: yaml
 
     httpd:
-      service:
-        - running
+      service.running: []
 
 The service can also be set to be started at runtime via the enable option:
 
 .. code-block:: yaml
 
     openvpn:
-      service:
-        - running
+      service.running:
         - enable: True
 
 By default if a service is triggered to refresh due to a watch statement the
-service is by default restarted. If the desired behaviour is to reload the
+service is by default restarted. If the desired behavior is to reload the
 service, then set the reload value to True:
 
 .. code-block:: yaml
 
     redis:
-      service:
-        - running
+      service.running:
         - enable: True
         - reload: True
         - watch:
           - pkg: redis
+
+.. note::
+
+    More details regarding ``watch`` can be found in the
+    :doc:`Requisites </ref/states/requisites>` documentation.
+
 '''
 
 
@@ -42,10 +45,13 @@ def __virtual__():
     Only make these states available if a service provider has been detected or
     assigned for this minion
     '''
-    return 'service' if 'service.start' in __salt__ else False
+    return 'service.start' in __salt__
 
 
 def _enabled_used_error(ret):
+    '''
+    Warn of potential typo.
+    '''
     ret['result'] = False
     ret['comment'] = (
         'Service {0} uses non-existent option "enabled".  ' +
@@ -58,47 +64,39 @@ def _enable(name, started, result=True, **kwargs):
     '''
     Enable the service
     '''
-    ret = {'name': name,
-           'changes': {},
-           'result': result,
-           'comment': ''}
+    ret = {}
 
     # is service available?
     if not _available(name, ret):
         return ret
 
     # Check to see if this minion supports enable
-    if not 'service.enable' in __salt__ or not 'service.enabled' in __salt__:
+    if 'service.enable' not in __salt__ or 'service.enabled' not in __salt__:
         if started is True:
             ret['comment'] = ('Enable is not available on this minion,'
                               ' service {0} started').format(name)
-            return ret
         elif started is None:
             ret['comment'] = ('Enable is not available on this minion,'
                               ' service {0} is in the desired state'
                               ).format(name)
-            return ret
         else:
             ret['comment'] = ('Enable is not available on this minion,'
                               ' service {0} is dead').format(name)
-            return ret
+        return ret
 
     # Service can be enabled
     if __salt__['service.enabled'](name):
         # Service is enabled
         if started is True:
-            ret['changes'][name] = True
             ret['comment'] = ('Service {0} is already enabled,'
                               ' and is running').format(name)
-            return ret
         elif started is None:
             ret['comment'] = ('Service {0} is already enabled,'
                               ' and is in the desired state').format(name)
-            return ret
         else:
             ret['comment'] = ('Service {0} is already enabled,'
                               ' and is dead').format(name)
-            return ret
+        return ret
 
     # Service needs to be enabled
     if __opts__['test']:
@@ -108,49 +106,40 @@ def _enable(name, started, result=True, **kwargs):
 
     if __salt__['service.enable'](name, **kwargs):
         # Service has been enabled
+        ret['changes'] = {}
+        ret['changes'][name] = True
         if started is True:
-            ret['changes'][name] = True
             ret['comment'] = ('Service {0} has been enabled,'
                               ' and is running').format(name)
-            return ret
         elif started is None:
-            ret['changes'][name] = True
             ret['comment'] = ('Service {0} has been enabled,'
                               ' and is in the desired state').format(name)
-            return ret
         else:
-            ret['changes'][name] = True
             ret['comment'] = ('Service {0} has been enabled,'
                               ' and is dead').format(name)
-            return ret
+        return ret
 
     # Service failed to be enabled
     if started is True:
-        ret['changes'][name] = True
         ret['result'] = False
         ret['comment'] = ('Failed when setting service {0} to start at boot,'
                           ' but the service is running').format(name)
-        return ret
     elif started is None:
         ret['result'] = False
         ret['comment'] = ('Failed when setting service {0} to start at boot,'
                           ' but the service was already running').format(name)
-        return ret
     else:
         ret['result'] = False
         ret['comment'] = ('Failed when setting service {0} to start at boot,'
                           ' and the service is dead').format(name)
-        return ret
+    return ret
 
 
 def _disable(name, started, result=True, **kwargs):
     '''
     Disable the service
     '''
-    ret = {'name': name,
-           'changes': {},
-           'result': result,
-           'comment': ''}
+    ret = {}
 
     # is service available?
     if not _available(name, ret):
@@ -158,37 +147,32 @@ def _disable(name, started, result=True, **kwargs):
         return ret
 
     # is enable/disable available?
-    if not 'service.disable' in __salt__ or not 'service.disabled' in __salt__:
+    if 'service.disable' not in __salt__ or 'service.disabled' not in __salt__:
         if started is True:
             ret['comment'] = ('Disable is not available on this minion,'
                               ' service {0} started').format(name)
-            return ret
         elif started is None:
             ret['comment'] = ('Disable is not available on this minion,'
                               ' service {0} is in the desired state'
                               ).format(name)
-            return ret
         else:
             ret['comment'] = ('Disable is not available on this minion,'
                               ' service {0} is dead').format(name)
-            return ret
+        return ret
 
     # Service can be disabled
     if __salt__['service.disabled'](name):
         # Service is disabled
         if started is True:
-            ret['changes'][name] = True
             ret['comment'] = ('Service {0} is already disabled,'
                               ' and is running').format(name)
-            return ret
         elif started is None:
             ret['comment'] = ('Service {0} is already disabled,'
                               ' and is in the desired state').format(name)
-            return ret
         else:
             ret['comment'] = ('Service {0} is already disabled,'
                               ' and is dead').format(name)
-            return ret
+        return ret
 
     # Service needs to be disabled
     if __opts__['test']:
@@ -198,44 +182,39 @@ def _disable(name, started, result=True, **kwargs):
 
     if __salt__['service.disable'](name, **kwargs):
         # Service has been disabled
+        ret['changes'] = {}
+        ret['changes'][name] = True
         if started is True:
-            ret['changes'][name] = True
             ret['comment'] = ('Service {0} has been disabled,'
                               ' and is running').format(name)
-            return ret
         elif started is None:
-            ret['changes'][name] = True
             ret['comment'] = ('Service {0} has been disabled,'
                               ' and is in the desired state').format(name)
-            return ret
         else:
-            ret['changes'][name] = True
             ret['comment'] = ('Service {0} has been disabled,'
                               ' and is dead').format(name)
-            return ret
+        return ret
 
     # Service failed to be disabled
+    ret['result'] = False
     if started is True:
-        ret['result'] = False
         ret['comment'] = ('Failed when setting service {0} to not start'
                           ' at boot, and is running').format(name)
-        return ret
     elif started is None:
-        ret['result'] = False
         ret['comment'] = ('Failed when setting service {0} to not start'
                           ' at boot, but the service was already running'
                           ).format(name)
         return ret
     else:
-        ret['changes'][name] = True
-        ret['result'] = False
         ret['comment'] = ('Failed when setting service {0} to not start'
                           ' at boot, and the service is dead').format(name)
-        return ret
+    return ret
 
 
 def _available(name, ret):
-    # Check if the service is available
+    '''
+    Check if the service is available
+    '''
     avail = False
     if 'service.available' in __salt__:
         avail = __salt__['service.available'](name)
@@ -279,9 +258,11 @@ def running(name, enable=None, sig=None, **kwargs):
     if __salt__['service.status'](name, sig):
         ret['comment'] = 'The service {0} is already running'.format(name)
         if enable is True:
-            return _enable(name, None, **kwargs)
+            ret.update(_enable(name, None, **kwargs))
+            return ret
         elif enable is False:
-            return _disable(name, None, **kwargs)
+            ret.update(_disable(name, None, **kwargs))
+            return ret
         else:
             return ret
 
@@ -291,26 +272,22 @@ def running(name, enable=None, sig=None, **kwargs):
         ret['comment'] = 'Service {0} is set to start'.format(name)
         return ret
 
-    changes = {name: __salt__['service.start'](name)}
+    ret['changes'] = {name: __salt__['service.start'](name)}
 
-    if not changes[name]:
+    if not ret['changes'][name]:
+        ret['result'] = False
+        ret['comment'] = 'Service {0} failed to start'.format(name)
         if enable is True:
-            return _enable(name, False, result=False, **kwargs)
+            ret.update(_enable(name, False, result=False, **kwargs))
         elif enable is False:
-            return _disable(name, False, result=False, **kwargs)
-        else:
-            ret['result'] = False
-            ret['comment'] = 'Service {0} failed to start'.format(name)
-            return ret
-
-    if enable is True:
-        return _enable(name, True, **kwargs)
-    elif enable is False:
-        return _disable(name, True, **kwargs)
+            ret.update(_disable(name, False, result=False, **kwargs))
     else:
-        ret['changes'] = changes
         ret['comment'] = 'Started Service {0}'.format(name)
-        return ret
+        if enable is True:
+            ret.update(_enable(name, True, **kwargs))
+        elif enable is False:
+            ret.update(_disable(name, True, **kwargs))
+    return ret
 
 
 def dead(name, enable=None, sig=None, **kwargs):
@@ -345,11 +322,10 @@ def dead(name, enable=None, sig=None, **kwargs):
     if not __salt__['service.status'](name, sig):
         ret['comment'] = 'The service {0} is already dead'.format(name)
         if enable is True:
-            return _enable(name, None, **kwargs)
+            ret.update(_enable(name, None, **kwargs))
         elif enable is False:
-            return _disable(name, None, **kwargs)
-        else:
-            return ret
+            ret.update(_disable(name, None, **kwargs))
+        return ret
 
     if __opts__['test']:
         ret['result'] = None
@@ -362,21 +338,16 @@ def dead(name, enable=None, sig=None, **kwargs):
         ret['result'] = False
         ret['comment'] = 'Service {0} failed to die'.format(name)
         if enable is True:
-            return _enable(name, True, result=False)
+            ret.update(_enable(name, True, result=False, **kwargs))
         elif enable is False:
-            return _disable(name, True, result=False)
-        else:
-            ret['result'] = False
-            ret['comment'] = 'Service {0} failed to die'.format(name)
-            return ret
+            ret.update(_disable(name, True, result=False, **kwargs))
     else:
+        ret['comment'] = 'Service {0} was killed'.format(name)
         if enable is True:
-            return _enable(name, False)
+            ret.update(_enable(name, False, **kwargs))
         elif enable is False:
-            return _disable(name, False)
-        else:
-            ret['comment'] = 'Service {0} was killed'.format(name)
-            return ret
+            ret.update(_disable(name, False, **kwargs))
+    return ret
 
 
 def enabled(name, **kwargs):
@@ -389,7 +360,13 @@ def enabled(name, **kwargs):
     name
         The name of the init or rc script used to manage the service
     '''
-    return _enable(name, None, **kwargs)
+    ret = {'name': name,
+           'changes': {},
+           'result': True,
+           'comment': ''}
+
+    ret.update(_enable(name, None, **kwargs))
+    return ret
 
 
 def disabled(name, **kwargs):
@@ -402,7 +379,13 @@ def disabled(name, **kwargs):
     name
         The name of the init or rc script used to manage the service
     '''
-    return _disable(name, None, **kwargs)
+    ret = {'name': name,
+           'changes': {},
+           'result': True,
+           'comment': ''}
+
+    ret.update(_disable(name, None, **kwargs))
+    return ret
 
 
 def mod_watch(name, sfun=None, sig=None, reload=False, full_restart=False):
@@ -414,8 +397,7 @@ def mod_watch(name, sfun=None, sig=None, reload=False, full_restart=False):
 
     sfun
         The original function which triggered the mod_watch call
-        (`service.running`, for example).  Currently not used, but must be
-        supported for the future.
+        (`service.running`, for example).
 
     sig
         The string to search for when looking for the service process with ps
@@ -424,31 +406,48 @@ def mod_watch(name, sfun=None, sig=None, reload=False, full_restart=False):
            'changes': {},
            'result': True,
            'comment': ''}
-    action = ''
 
-    if __salt__['service.status'](name, sig):
-        if 'service.reload' in __salt__ and reload:
-            restart_func = __salt__['service.reload']
-            action = 'reload'
-        elif 'service.full_restart' in __salt__ and full_restart:
-            restart_func = __salt__['service.full_restart']
-            action = 'fully restart'
+    if sfun == 'dead':
+        verb = 'stop'
+        past_participle = verb + 'ped'
+        if __salt__['service.status'](name, sig):
+            func = __salt__['service.stop']
         else:
-            restart_func = __salt__['service.restart']
-            action = 'restart'
+            ret['result'] = True
+            ret['comment'] = 'Service is already {0}'.format(past_participle)
+            return ret
+    elif sfun == 'running':
+        if __salt__['service.status'](name, sig):
+            if 'service.reload' in __salt__ and reload:
+                func = __salt__['service.reload']
+                verb = 'reload'
+                past_participle = verb + 'ed'
+            elif 'service.full_restart' in __salt__ and full_restart:
+                func = __salt__['service.full_restart']
+                verb = 'fully restart'
+                past_participle = verb + 'ed'
+            else:
+                func = __salt__['service.restart']
+                verb = 'restart'
+                past_participle = verb + 'ed'
+        else:
+            func = __salt__['service.start']
+            verb = 'start'
+            past_participle = verb + 'ed'
     else:
-        restart_func = __salt__['service.start']
-        action = 'start'
+        ret['comment'] = 'Unable to trigger watch for service.{0}'.format(sfun)
+        ret['result'] = False
+        return ret
 
     if __opts__['test']:
         ret['result'] = None
-        ret['comment'] = 'Service is set to be {0}ed'.format(action)
+        ret['comment'] = 'Service is set to be {0}'.format(past_participle)
         return ret
 
-    result = restart_func(name)
+    result = func(name)
 
     ret['changes'] = {name: result}
     ret['result'] = result
-    ret['comment'] = 'Service {0}ed'.format(action) if result else \
-                     'Failed to {0} the service'.format(action)
+    ret['comment'] = 'Service {0}'.format(past_participle) if result else \
+                     'Failed to {0} the service'.format(verb)
     return ret
