@@ -6,8 +6,7 @@ Interaction with the Supervisor daemon
 .. code-block:: yaml
 
     wsgi_server:
-      supervisord:
-        - running
+      supervisord.running:
         - require:
           - pkg: supervisor
         - watch:
@@ -27,8 +26,13 @@ def _check_error(result, success_message):
     ret = {}
 
     if 'ERROR' in result:
-        ret['comment'] = result
-        ret['result'] = False
+        if 'already started' in result:
+            ret['comment'] = success_message
+        elif 'not running' in result:
+            ret['comment'] = success_message
+        else:
+            ret['comment'] = result
+            ret['result'] = False
     else:
         ret['comment'] = success_message
 
@@ -79,7 +83,7 @@ def running(name,
     ret = {'name': name, 'result': True, 'comment': '', 'changes': {}}
 
     salt.utils.warn_until(
-        'Hydrogen',
+        'Lithium',
         'Please remove \'runas\' support at this stage. \'user\' support was '
         'added in 0.17.0',
         _dont_call_warnings=True
@@ -102,6 +106,11 @@ def running(name,
         user = runas
         runas = None
 
+    if 'supervisord.status' not in __salt__:
+        ret['result'] = False
+        ret['comment'] = 'Supervisord module not activated. Do you need to install supervisord?'
+        return ret
+
     all_processes = __salt__['supervisord.status'](
         user=user,
         conf_file=conf_file,
@@ -109,27 +118,66 @@ def running(name,
     )
 
     # parse process groups
-    process_groups = []
+    process_groups = set()
     for proc in all_processes:
         if ':' in proc:
-            process_groups.append(proc[:proc.index(':') + 1])
-    process_groups = list(set(process_groups))
+            process_groups.add(proc[:proc.index(':') + 1])
+    process_groups = sorted(process_groups)
 
-    # determine if this process/group needs loading
-    needs_update = name not in all_processes and name not in process_groups
+    matches = {}
+    if name in all_processes:
+        matches[name] = (all_processes[name]['state'].lower() == 'running')
+    elif name in process_groups:
+        for process in (x for x in all_processes if x.startswith(name)):
+            matches[process] = (
+                all_processes[process]['state'].lower() == 'running'
+            )
+    to_add = not bool(matches)
 
     if __opts__['test']:
-        ret['result'] = None
-        _msg = 'restarted' if restart else 'started'
-        _update = ', but service needs to be added' if needs_update else ''
-        ret['comment'] = (
-            'Service {0} is set to be {1}{2}'.format(
-                name, _msg, _update))
+        if not to_add:
+            # Process/group already present, check if any need to be started
+            to_start = [x for x, y in matches.iteritems() if y is False]
+            if to_start:
+                ret['result'] = None
+                if name.endswith(':'):
+                    # Process group
+                    if len(to_start) == len(matches):
+                        ret['comment'] = (
+                            'All services in group {0!r} will be started'
+                            .format(name)
+                        )
+                    else:
+                        ret['comment'] = (
+                            'The following services will be started: {0}'
+                            .format(' '.join(to_start))
+                        )
+                else:
+                    # Single program
+                    ret['comment'] = 'Service {0} will be started'.format(name)
+            else:
+                if name.endswith(':'):
+                    # Process group
+                    ret['comment'] = (
+                        'All services in group {0!r} are already running'
+                        .format(name)
+                    )
+                else:
+                    ret['comment'] = ('Service {0} is already running'
+                                      .format(name))
+        else:
+            ret['result'] = None
+            # Process/group needs to be added
+            if name.endswith(':'):
+                _type = 'Group {0!r}'.format(name)
+            else:
+                _type = 'Service {0}'.format(name)
+            ret['comment'] = '{0} will be added and started'.format(_type)
         return ret
 
     changes = []
     just_updated = False
-    if needs_update:
+    if to_add:
         comment = 'Adding service: {0}'.format(name)
         __salt__['supervisord.reread'](
             user=user,
@@ -155,7 +203,6 @@ def running(name,
             bin_env=bin_env
         )
         ret.update(_check_error(result, comment))
-        changes.append(comment)
         log.debug(comment)
 
         if '{0}: updated'.format(name) in result:
@@ -267,7 +314,7 @@ def dead(name,
     ret = {'name': name, 'result': True, 'comment': '', 'changes': {}}
 
     salt.utils.warn_until(
-        'Hydrogen',
+        'Lithium',
         'Please remove \'runas\' support at this stage. \'user\' support was '
         'added in 0.17.0',
         _dont_call_warnings=True
