@@ -10,6 +10,7 @@ import os
 import re
 import sys
 import stat
+import errno
 import socket
 import logging
 
@@ -22,6 +23,7 @@ else:
 # Import salt libs
 from salt.log import is_console_configured
 from salt.exceptions import SaltClientError
+import salt.exitcodes
 import salt.utils
 
 log = logging.getLogger(__name__)
@@ -156,12 +158,15 @@ def verify_files(files, user):
         err = ('Failed to prepare the Salt environment for user '
                '{0}. The user is not available.\n').format(user)
         sys.stderr.write(err)
-        sys.exit(os.EX_NOUSER)
+        sys.exit(salt.exitcodes.EX_NOUSER)
     for fn_ in files:
         dirname = os.path.dirname(fn_)
         try:
-            if not os.path.isdir(dirname):
+            try:
                 os.makedirs(dirname)
+            except OSError as err:
+                if err.errno != errno.EEXIST:
+                    raise
             if not os.path.isfile(fn_):
                 with salt.utils.fopen(fn_, 'w+') as fp_:
                     fp_.write('')
@@ -197,7 +202,7 @@ def verify_env(dirs, user, permissive=False, pki_dir=''):
         err = ('Failed to prepare the Salt environment for user '
                '{0}. The user is not available.\n').format(user)
         sys.stderr.write(err)
-        sys.exit(os.EX_NOUSER)
+        sys.exit(salt.exitcodes.EX_NOUSER)
     for dir_ in dirs:
         if not dir_:
             continue
